#!/usr/bin/perl
# test/t_flash.pl

=head1 t_flash.pl - test Flash XML handler

=cut

use warnings;
use strict;

use Test::More qw(no_plan);

use XML::Simple qw/:strict/;
use LWP::UserAgent;
use HTTP::Request::Common;
use HTTP::Cookies;
use DBI;
use Carp;

my $server = 'http://centos41';
my $user_login = 'user@somedomain.com';
my $user_password = 'user';
my ($message, $response, $expect);
my $dbh;
my $ua;
my $cookie_jar;
my $sample_image_b64 = <<EOM ;
R0lGODlhFAAWAOMAAP////+Zmf8zM8z//8zMzJmZmWYAADMzMwAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAACH+TlRoaXMgYXJ0IGlzIGluIHRoZSBwdWJsaWMgZG9tYWluLiBLZXZpbiBIdWdoZXMsIGtl
dmluaEBlaXQuY29tLCBTZXB0ZW1iZXIgMTk5NQAh+QQBAAADACwAAAAAFAAWAAAEeHDISau9OOuN
OtoUEozCB4pAIAiGmYlBqrLuhQiqauz1dBQFEW5FQwCBB8kBALgRdy0EExBUMp0GVlTKrA6WV2ir
NaWawGEPd+pFXz3l7pl9rJfbTAKhfiSw53l7ggV+f1ZTeomFhl9xjneAj45tP3yWSDVqmpsmEQA7
Cg==
EOM

{
  $cookie_jar = HTTP::Cookies->new( file => './cookies.dat', ignore_discard => 1, autosave => 1 );
  $ua = LWP::UserAgent->new( agent => 'perl post' );
  $ua->cookie_jar($cookie_jar);

	$dbh = DBI->connect("DBI:mysql:app", "app", "app") ||confess "cannot connect to mysql database";

  login();
  t_register();
  t_login();
  t_reset_password();
  t_picture();

  $dbh->disconnect;
  $dbh = undef;
}

#===

sub postit
{ 
  my $message = shift;
  my $not_logged_in = shift;
  if ( $not_logged_in )
  {
    $cookie_jar->save;
    $cookie_jar->clear;
  }  
  #"<?xml version=\"1.0\" encoding=\"UTF-8\"?>
  my $response = $ua->request(POST
    $server.'/app/flash',
    Content_Type => 'text/xml',
    Content => $message
    );
  if ( $not_logged_in )
  {
    $cookie_jar->revert;
  }
  else
  {
    $cookie_jar->extract_cookies($response);
    $cookie_jar->save;
  }
  $response;
}

sub do_sql
{
	my $s = shift || confess;
	$s =~ s/[\000-\037]/ /g; # nonprint to space
	$s =~ s/;[\s]*$//; # no end ;
	my $sth = $dbh->prepare( $s ) || confess "cannot prepare sql: $dbh->errstr";
	$sth->execute(@_) || confess "cannot execute sql: " . $dbh->errstr;
	#print $sth->rows, " rows affected\n";
}

sub login
{
  $message = '<request cmd="login" user_email="'.$user_login.'" user_password="'.$user_password.'" />';
  $expect = qq{<response cmd="login" result="OK"};
  $response = postit($message);
  confess "cannot login" unless $response->as_string =~ m/$expect/;
  diag 'logged in';
}

sub logout
{
  $message = '<request cmd="logout" />';
  $expect = qq{<response cmd="logout" result="OK"};
  $response = postit($message);
  confess "cannot logout" unless $response->as_string =~ m/$expect/;
  diag 'logged out';
}

#===


sub t_register
{
  do_sql('delete from user where login=?', 'user2@somedomain.com');
  $message = '<request cmd="register" user_email="user2@somedomain.com" user_password="user" user_name="A User" />';
  $expect = qq{<response cmd="register" result="OK"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'register - new name gives OK' ) || print $response->as_string;

  $message = '<request cmd="register" user_email="user2@somedomain.com" user_password="test" user_name="A 2nd User" />';
  $expect = qq{<response cmd="register" result="NOK"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'register - existing name gives NOK' ) || print $response->as_string;
}

sub t_login
{
  $message = '<request cmd="login_wrong_command" user_email="'.$user_login.'" user_password="'.$user_password.'" />';
  $expect = qq{Flash handler unknown cmd};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'invalid command gives Flash handler error' ) || print $response->as_string;

  $message = '<request cmd="login" user_email="fred@bloggs.com" user_password="bloggs" />';
  $expect = qq{<response cmd="login" result="NOK" reason="invalid user/password"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'login - invalid username/password gives NOK' ) || print $response->as_string;

  $message = '<request cmd="login" user_email="'.$user_login.'" user_password="'.$user_password.'" />';
  $expect = qq{<response cmd="login" result="OK"};
  $response = postit($message);
  ok ( $response->is_success, 'Flash XML handler responds to known command login' );
  ok ( ($response->as_string =~ m/$expect/), 'login - valid username/password gives OK' ) || print $response->as_string;
}

sub t_reset_password
{
  $message = '<request cmd="reset_password" user_email="user2@somedomain.com" />';
  $expect = qq{<response cmd="reset_password" result="OK"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'reset_password' ) || print $response->as_string;
}

sub t_picture
{
  logout();
  $message = '<request cmd="picture_upload" description="test image" name="Fred Bloggs" location="Bloggstown" collection_type="photo" />';
  $expect = qq{<response cmd="picture_upload" result="NOK" reason="not logged in"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'picture_upload - not logged in gives NOK' ) || print $response->as_string;
  login();

  $message = '<request cmd="picture_upload" photo_image="'.$sample_image_b64.'" description="test image" name="Fred Bloggs" location="Bloggstown" collection_type="photo" />';
  $expect = qq{<response cmd="picture_upload" result="OK"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'picture_upload' ) || print $response->as_string;

  $message = '<request cmd="picture_list" collection_type="photo" />';
  $expect = qq{<response cmd="picture_list" result="OK"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'picture_list' ) || print $response->as_string;

  $message = '<request cmd="picture_get" />';
  $expect = qq{<response cmd="picture_get" result="NOK" reason="expected parameters};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'picture_get - missing id gives NOK expected parameters' ) || print $response->as_string;

  $message = '<request cmd="picture_get" id="1" />';
  $expect = qq{<response cmd="picture_get" result="OK"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'picture_get - id 1 gives OK' ) || print $response->as_string;

  $message = '<request cmd="picture_get" id="999999999" />';
  $expect = qq{<response cmd="picture_get" result="NOK" reason="not found"};
  $response = postit($message);
  ok ( ($response->as_string =~ m/$expect/), 'picture_get - id 999999999 gives NOK not found' ) || print $response->as_string;
}

__END__
